@@ -5,120 +5,128 @@
 namespace esphome {
 namespace dallas {
 
-static const char *TAG = "dallas.one_wire";
+static const char *const TAG = "dallas.one_wire";
 
 const uint8_t ONE_WIRE_ROM_SELECT = 0x55;
 const int ONE_WIRE_ROM_SEARCH = 0xF0;
 
-ESPOneWire::ESPOneWire(GPIOPin *pin) : pin_(pin) {}
+ESPOneWire::ESPOneWire(InternalGPIOPin *pin) { pin_ = pin->to_isr(); }
 
-bool HOT ICACHE_RAM_ATTR ESPOneWire::reset() {
+bool HOT IRAM_ATTR ESPOneWire::reset() {
+  // See reset here:
+  // https://www.maximintegrated.com/en/design/technical-documents/app-notes/1/126.html
+  InterruptLock lock;
+
+  // Wait for communication to clear (delay G)
+  pin_.pin_mode(gpio::FLAG_INPUT | gpio::FLAG_PULLUP);
   uint8_t retries = 125;
-
-  // Wait for communication to clear
-  this->pin_->pin_mode(INPUT_PULLUP);
   do {
     if (--retries == 0)
       return false;
     delayMicroseconds(2);
-  } while (!this->pin_->digital_read());
+  } while (!pin_.digital_read());
 
-  // Send 480µs LOW TX reset pulse
-  this->pin_->pin_mode(OUTPUT);
-  this->pin_->digital_write(false);
+  // Send 480µs LOW TX reset pulse (drive bus low, delay H)
+  pin_.pin_mode(gpio::FLAG_OUTPUT);
+  pin_.digital_write(false);
   delayMicroseconds(480);
 
-  // Switch into RX mode, letting the pin float
-  this->pin_->pin_mode(INPUT_PULLUP);
-  // after 15µs-60µs wait time, slave pulls low for 60µs-240µs
-  // let's have 70µs just in case
+  // Release the bus, delay I
+  pin_.pin_mode(gpio::FLAG_INPUT | gpio::FLAG_PULLUP);
   delayMicroseconds(70);
 
-  bool r = !this->pin_->digital_read();
+  // sample bus, 0=device(s) present, 1=no device present
+  bool r = !pin_.digital_read();
+  // delay J
   delayMicroseconds(410);
   return r;
 }
 
-void HOT ICACHE_RAM_ATTR ESPOneWire::write_bit(bool bit) {
-  // Initiate write/read by pulling low.
-  this->pin_->pin_mode(OUTPUT);
-  this->pin_->digital_write(false);
+void HOT IRAM_ATTR ESPOneWire::write_bit(bool bit) {
+  // See write 1/0 bit here:
+  // https://www.maximintegrated.com/en/design/technical-documents/app-notes/1/126.html
+  InterruptLock lock;
 
-  // bus sampled within 15µs and 60µs after pulling LOW.
-  if (bit) {
-    // pull high/release within 15µs
-    delayMicroseconds(10);
-    this->pin_->digital_write(true);
-    // in total minimum of 60µs long
-    delayMicroseconds(55);
-  } else {
-    // continue pulling LOW for at least 60µs
-    delayMicroseconds(65);
-    this->pin_->digital_write(true);
-    // grace period, 1µs recovery time
-    delayMicroseconds(5);
-  }
+  // drive bus low
+  pin_.pin_mode(gpio::FLAG_OUTPUT);
+  pin_.digital_write(false);
+
+  uint32_t delay0 = bit ? 10 : 65;
+  uint32_t delay1 = bit ? 55 : 5;
+
+  // delay A/C
+  delayMicroseconds(delay0);
+  // release bus
+  pin_.digital_write(true);
+  // delay B/D
+  delayMicroseconds(delay1);
 }
 
-bool HOT ICACHE_RAM_ATTR ESPOneWire::read_bit() {
-  // Initiate read slot by pulling LOW for at least 1µs
-  this->pin_->pin_mode(OUTPUT);
-  this->pin_->digital_write(false);
+bool HOT IRAM_ATTR ESPOneWire::read_bit() {
+  // See read bit here:
+  // https://www.maximintegrated.com/en/design/technical-documents/app-notes/1/126.html
+  InterruptLock lock;
+
+  // drive bus low, delay A
+  pin_.pin_mode(gpio::FLAG_OUTPUT);
+  pin_.digital_write(false);
   delayMicroseconds(3);
 
-  // release bus, we have to sample within 15µs of pulling low
-  this->pin_->pin_mode(INPUT_PULLUP);
+  // release bus, delay E
+  pin_.pin_mode(gpio::FLAG_INPUT | gpio::FLAG_PULLUP);
   delayMicroseconds(10);
 
-  bool r = this->pin_->digital_read();
-  // read time slot at least 60µs long + 1µs recovery time between slots
+  // sample bus to read bit from peer
+  bool r = pin_.digital_read();
+
+  // delay F
   delayMicroseconds(53);
   return r;
 }
 
-void ICACHE_RAM_ATTR ESPOneWire::write8(uint8_t val) {
+void ESPOneWire::write8(uint8_t val) {
   for (uint8_t i = 0; i < 8; i++) {
     this->write_bit(bool((1u << i) & val));
   }
 }
 
-void ICACHE_RAM_ATTR ESPOneWire::write64(uint64_t val) {
+void ESPOneWire::write64(uint64_t val) {
   for (uint8_t i = 0; i < 64; i++) {
     this->write_bit(bool((1ULL << i) & val));
   }
 }
 
-uint8_t ICACHE_RAM_ATTR ESPOneWire::read8() {
+uint8_t ESPOneWire::read8() {
   uint8_t ret = 0;
   for (uint8_t i = 0; i < 8; i++) {
     ret |= (uint8_t(this->read_bit()) << i);
   }
   return ret;
 }
-uint64_t ICACHE_RAM_ATTR ESPOneWire::read64() {
+uint64_t ESPOneWire::read64() {
   uint64_t ret = 0;
   for (uint8_t i = 0; i < 8; i++) {
     ret |= (uint64_t(this->read_bit()) << i);
   }
   return ret;
 }
-void ICACHE_RAM_ATTR ESPOneWire::select(uint64_t address) {
+void ESPOneWire::select(uint64_t address) {
   this->write8(ONE_WIRE_ROM_SELECT);
   this->write64(address);
 }
-void ICACHE_RAM_ATTR ESPOneWire::reset_search() {
+void ESPOneWire::reset_search() {
   this->last_discrepancy_ = 0;
   this->last_device_flag_ = false;
   this->last_family_discrepancy_ = 0;
   this->rom_number_ = 0;
 }
-uint64_t HOT ICACHE_RAM_ATTR ESPOneWire::search() {
+uint64_t ESPOneWire::search() {
   if (this->last_device_flag_) {
     return 0u;
   }
 
   if (!this->reset()) {
-    // Reset failed
+    // Reset failed or no devices present
     this->reset_search();
     return 0u;
   }
@@ -137,9 +145,10 @@ uint64_t HOT ICACHE_RAM_ATTR ESPOneWire::search() {
     // read its complement
     bool cmp_id_bit = this->read_bit();
 
-    if (id_bit && cmp_id_bit)
+    if (id_bit && cmp_id_bit) {
       // No devices participating in search
       break;
+    }
 
     bool branch;
 
@@ -162,12 +171,13 @@ uint64_t HOT ICACHE_RAM_ATTR ESPOneWire::search() {
       }
     }
 
-    if (branch)
+    if (branch) {
       // set bit
       this->rom_number8_()[rom_byte_number] |= rom_byte_mask;
-    else
+    } else {
       // clear bit
       this->rom_number8_()[rom_byte_number] &= ~rom_byte_mask;
+    }
 
     // choose/announce branch
     this->write_bit(branch);
@@ -182,9 +192,10 @@ uint64_t HOT ICACHE_RAM_ATTR ESPOneWire::search() {
 
   if (id_bit_number >= 65) {
     this->last_discrepancy_ = last_zero;
-    if (this->last_discrepancy_ == 0)
+    if (this->last_discrepancy_ == 0) {
       // we're at root and have no choices left, so this was the last one.
       this->last_device_flag_ = true;
+    }
     search_result = true;
   }
 
@@ -196,7 +207,7 @@ uint64_t HOT ICACHE_RAM_ATTR ESPOneWire::search() {
 
   return this->rom_number_;
 }
-std::vector<uint64_t> ICACHE_RAM_ATTR ESPOneWire::search_vec() {
+std::vector<uint64_t> ESPOneWire::search_vec() {
   std::vector<uint64_t> res;
 
   this->reset_search();
@@ -206,12 +217,11 @@ std::vector<uint64_t> ICACHE_RAM_ATTR ESPOneWire::search_vec() {
 
   return res;
 }
-void ICACHE_RAM_ATTR ESPOneWire::skip() {
+void ESPOneWire::skip() {
   this->write8(0xCC);  // skip ROM
 }
-GPIOPin *ESPOneWire::get_pin() { return this->pin_; }
 
-uint8_t ICACHE_RAM_ATTR *ESPOneWire::rom_number8_() { return reinterpret_cast<uint8_t *>(&this->rom_number_); }
+uint8_t IRAM_ATTR *ESPOneWire::rom_number8_() { return reinterpret_cast<uint8_t *>(&this->rom_number_); }
 
 }  // namespace dallas
 }  // namespace esphome
