@@ -6,7 +6,7 @@ namespace sensor {
 
 static const char *const TAG = "sensor";
 
-const char *state_class_to_string(StateClass state_class) {
+std::string state_class_to_string(StateClass state_class) {
   switch (state_class) {
     case STATE_CLASS_MEASUREMENT:
       return "measurement";
@@ -18,17 +18,54 @@ const char *state_class_to_string(StateClass state_class) {
   }
 }
 
-const char *last_reset_type_to_string(LastResetType last_reset_type) {
-  switch (last_reset_type) {
-    case LAST_RESET_TYPE_NEVER:
-      return "never";
-    case LAST_RESET_TYPE_AUTO:
-      return "auto";
-    case LAST_RESET_TYPE_NONE:
-    default:
-      return "";
-  }
+Sensor::Sensor(const std::string &name) : EntityBase(name), state(NAN), raw_state(NAN) {}
+Sensor::Sensor() : Sensor("") {}
+
+std::string Sensor::get_unit_of_measurement() {
+  if (this->unit_of_measurement_.has_value())
+    return *this->unit_of_measurement_;
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+  return this->unit_of_measurement();
+#pragma GCC diagnostic pop
 }
+void Sensor::set_unit_of_measurement(const std::string &unit_of_measurement) {
+  this->unit_of_measurement_ = unit_of_measurement;
+}
+std::string Sensor::unit_of_measurement() { return ""; }
+
+int8_t Sensor::get_accuracy_decimals() {
+  if (this->accuracy_decimals_.has_value())
+    return *this->accuracy_decimals_;
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+  return this->accuracy_decimals();
+#pragma GCC diagnostic pop
+}
+void Sensor::set_accuracy_decimals(int8_t accuracy_decimals) { this->accuracy_decimals_ = accuracy_decimals; }
+int8_t Sensor::accuracy_decimals() { return 0; }
+
+std::string Sensor::get_device_class() {
+  if (this->device_class_.has_value())
+    return *this->device_class_;
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+  return this->device_class();
+#pragma GCC diagnostic pop
+}
+void Sensor::set_device_class(const std::string &device_class) { this->device_class_ = device_class; }
+std::string Sensor::device_class() { return ""; }
+
+void Sensor::set_state_class(StateClass state_class) { this->state_class_ = state_class; }
+StateClass Sensor::get_state_class() {
+  if (this->state_class_.has_value())
+    return *this->state_class_;
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+  return this->state_class();
+#pragma GCC diagnostic pop
+}
+StateClass Sensor::state_class() { return StateClass::STATE_CLASS_NONE; }
 
 void Sensor::publish_state(float state) {
   this->raw_state = state;
@@ -42,55 +79,12 @@ void Sensor::publish_state(float state) {
     this->filter_list_->input(state);
   }
 }
-std::string Sensor::unit_of_measurement() { return ""; }
-std::string Sensor::icon() { return ""; }
-uint32_t Sensor::update_interval() { return 0; }
-int8_t Sensor::accuracy_decimals() { return 0; }
-Sensor::Sensor(const std::string &name) : Nameable(name), state(NAN), raw_state(NAN) {}
-Sensor::Sensor() : Sensor("") {}
 
-void Sensor::set_unit_of_measurement(const std::string &unit_of_measurement) {
-  this->unit_of_measurement_ = unit_of_measurement;
-}
-void Sensor::set_icon(const std::string &icon) { this->icon_ = icon; }
-void Sensor::set_accuracy_decimals(int8_t accuracy_decimals) { this->accuracy_decimals_ = accuracy_decimals; }
 void Sensor::add_on_state_callback(std::function<void(float)> &&callback) { this->callback_.add(std::move(callback)); }
 void Sensor::add_on_raw_state_callback(std::function<void(float)> &&callback) {
   this->raw_callback_.add(std::move(callback));
 }
-std::string Sensor::get_icon() {
-  if (this->icon_.has_value())
-    return *this->icon_;
-  return this->icon();
-}
-void Sensor::set_device_class(const std::string &device_class) { this->device_class_ = device_class; }
-std::string Sensor::get_device_class() {
-  if (this->device_class_.has_value())
-    return *this->device_class_;
-  return this->device_class();
-}
-std::string Sensor::device_class() { return ""; }
-void Sensor::set_state_class(StateClass state_class) { this->state_class = state_class; }
-void Sensor::set_state_class(const std::string &state_class) {
-  if (str_equals_case_insensitive(state_class, "measurement")) {
-    this->state_class = STATE_CLASS_MEASUREMENT;
-  } else if (str_equals_case_insensitive(state_class, "total_increasing")) {
-    this->state_class = STATE_CLASS_TOTAL_INCREASING;
-  } else {
-    ESP_LOGW(TAG, "'%s' - Unrecognized state class %s", this->get_name().c_str(), state_class.c_str());
-  }
-}
-void Sensor::set_last_reset_type(LastResetType last_reset_type) { this->last_reset_type = last_reset_type; }
-std::string Sensor::get_unit_of_measurement() {
-  if (this->unit_of_measurement_.has_value())
-    return *this->unit_of_measurement_;
-  return this->unit_of_measurement();
-}
-int8_t Sensor::get_accuracy_decimals() {
-  if (this->accuracy_decimals_.has_value())
-    return *this->accuracy_decimals_;
-  return this->accuracy_decimals();
-}
+
 void Sensor::add_filter(Filter *filter) {
   // inefficient, but only happens once on every sensor setup and nobody's going to have massive amounts of
   // filters
@@ -132,24 +126,7 @@ void Sensor::internal_send_state_to_frontend(float state) {
   this->callback_.call(state);
 }
 bool Sensor::has_state() const { return this->has_state_; }
-uint32_t Sensor::calculate_expected_filter_update_interval() {
-  uint32_t interval = this->update_interval();
-  if (interval == 4294967295UL)
-    // update_interval: never
-    return 0;
-
-  if (this->filter_list_ == nullptr) {
-    return interval;
-  }
-
-  return this->filter_list_->calculate_remaining_interval(interval);
-}
 uint32_t Sensor::hash_base() { return 2455723294UL; }
 
-PollingSensorComponent::PollingSensorComponent(const std::string &name, uint32_t update_interval)
-    : PollingComponent(update_interval), Sensor(name) {}
-
-uint32_t PollingSensorComponent::update_interval() { return this->get_update_interval(); }
-
 }  // namespace sensor
 }  // namespace esphome
