@@ -1,8 +1,16 @@
 #pragma once
 
 #include "esphome/core/component.h"
-#include "esphome/core/esphal.h"
+#include "esphome/core/hal.h"
+#include <vector>
+
+#ifdef USE_ARDUINO
+#define USE_SPI_ARDUINO_BACKEND
+#endif
+
+#ifdef USE_SPI_ARDUINO_BACKEND
 #include <SPI.h>
+#endif
 
 namespace esphome {
 namespace spi {
@@ -72,18 +80,22 @@ class SPIComponent : public Component {
   void dump_config() override;
 
   template<SPIBitOrder BIT_ORDER, SPIClockPolarity CLOCK_POLARITY, SPIClockPhase CLOCK_PHASE> uint8_t read_byte() {
+#ifdef USE_SPI_ARDUINO_BACKEND
     if (this->hw_spi_ != nullptr) {
       return this->hw_spi_->transfer(0x00);
     }
+#endif  // USE_SPI_ARDUINO_BACKEND
     return this->transfer_<BIT_ORDER, CLOCK_POLARITY, CLOCK_PHASE, true, false>(0x00);
   }
 
   template<SPIBitOrder BIT_ORDER, SPIClockPolarity CLOCK_POLARITY, SPIClockPhase CLOCK_PHASE>
   void read_array(uint8_t *data, size_t length) {
+#ifdef USE_SPI_ARDUINO_BACKEND
     if (this->hw_spi_ != nullptr) {
       this->hw_spi_->transfer(data, length);
       return;
     }
+#endif  // USE_SPI_ARDUINO_BACKEND
     for (size_t i = 0; i < length; i++) {
       data[i] = this->read_byte<BIT_ORDER, CLOCK_POLARITY, CLOCK_PHASE>();
     }
@@ -91,19 +103,23 @@ class SPIComponent : public Component {
 
   template<SPIBitOrder BIT_ORDER, SPIClockPolarity CLOCK_POLARITY, SPIClockPhase CLOCK_PHASE>
   void write_byte(uint8_t data) {
+#ifdef USE_SPI_ARDUINO_BACKEND
     if (this->hw_spi_ != nullptr) {
       this->hw_spi_->write(data);
       return;
     }
+#endif  // USE_SPI_ARDUINO_BACKEND
     this->transfer_<BIT_ORDER, CLOCK_POLARITY, CLOCK_PHASE, false, true>(data);
   }
 
   template<SPIBitOrder BIT_ORDER, SPIClockPolarity CLOCK_POLARITY, SPIClockPhase CLOCK_PHASE>
   void write_byte16(const uint16_t data) {
+#ifdef USE_SPI_ARDUINO_BACKEND
     if (this->hw_spi_ != nullptr) {
       this->hw_spi_->write16(data);
       return;
     }
+#endif  // USE_SPI_ARDUINO_BACKEND
 
     this->write_byte<BIT_ORDER, CLOCK_POLARITY, CLOCK_PHASE>(data >> 8);
     this->write_byte<BIT_ORDER, CLOCK_POLARITY, CLOCK_PHASE>(data);
@@ -111,12 +127,14 @@ class SPIComponent : public Component {
 
   template<SPIBitOrder BIT_ORDER, SPIClockPolarity CLOCK_POLARITY, SPIClockPhase CLOCK_PHASE>
   void write_array16(const uint16_t *data, size_t length) {
+#ifdef USE_SPI_ARDUINO_BACKEND
     if (this->hw_spi_ != nullptr) {
       for (size_t i = 0; i < length; i++) {
         this->hw_spi_->write16(data[i]);
       }
       return;
     }
+#endif  // USE_SPI_ARDUINO_BACKEND
     for (size_t i = 0; i < length; i++) {
       this->write_byte16<BIT_ORDER, CLOCK_POLARITY, CLOCK_PHASE>(data[i]);
     }
@@ -124,11 +142,13 @@ class SPIComponent : public Component {
 
   template<SPIBitOrder BIT_ORDER, SPIClockPolarity CLOCK_POLARITY, SPIClockPhase CLOCK_PHASE>
   void write_array(const uint8_t *data, size_t length) {
+#ifdef USE_SPI_ARDUINO_BACKEND
     if (this->hw_spi_ != nullptr) {
       auto *data_c = const_cast<uint8_t *>(data);
       this->hw_spi_->writeBytes(data_c, length);
       return;
     }
+#endif  // USE_SPI_ARDUINO_BACKEND
     for (size_t i = 0; i < length; i++) {
       this->write_byte<BIT_ORDER, CLOCK_POLARITY, CLOCK_PHASE>(data[i]);
     }
@@ -136,6 +156,7 @@ class SPIComponent : public Component {
 
   template<SPIBitOrder BIT_ORDER, SPIClockPolarity CLOCK_POLARITY, SPIClockPhase CLOCK_PHASE>
   uint8_t transfer_byte(uint8_t data) {
+#ifdef USE_SPI_ARDUINO_BACKEND
     if (this->miso_ != nullptr) {
       if (this->hw_spi_ != nullptr) {
         return this->hw_spi_->transfer(data);
@@ -143,12 +164,14 @@ class SPIComponent : public Component {
         return this->transfer_<BIT_ORDER, CLOCK_POLARITY, CLOCK_PHASE, true, true>(data);
       }
     }
+#endif  // USE_SPI_ARDUINO_BACKEND
     this->write_byte<BIT_ORDER, CLOCK_POLARITY, CLOCK_PHASE>(data);
     return 0;
   }
 
   template<SPIBitOrder BIT_ORDER, SPIClockPolarity CLOCK_POLARITY, SPIClockPhase CLOCK_PHASE>
   void transfer_array(uint8_t *data, size_t length) {
+#ifdef USE_SPI_ARDUINO_BACKEND
     if (this->hw_spi_ != nullptr) {
       if (this->miso_ != nullptr) {
         this->hw_spi_->transfer(data, length);
@@ -157,6 +180,7 @@ class SPIComponent : public Component {
       }
       return;
     }
+#endif  // USE_SPI_ARDUINO_BACKEND
 
     if (this->miso_ != nullptr) {
       for (size_t i = 0; i < length; i++) {
@@ -169,18 +193,26 @@ class SPIComponent : public Component {
 
   template<SPIBitOrder BIT_ORDER, SPIClockPolarity CLOCK_POLARITY, SPIClockPhase CLOCK_PHASE, uint32_t DATA_RATE>
   void enable(GPIOPin *cs) {
-    if (cs != nullptr) {
-      SPIComponent::debug_enable(cs->get_pin());
-    }
-
+#ifdef USE_SPI_ARDUINO_BACKEND
     if (this->hw_spi_ != nullptr) {
-      uint8_t data_mode = (uint8_t(CLOCK_POLARITY) << 1) | uint8_t(CLOCK_PHASE);
+      uint8_t data_mode = SPI_MODE0;
+      if (!CLOCK_POLARITY && CLOCK_PHASE) {
+        data_mode = SPI_MODE1;
+      } else if (CLOCK_POLARITY && !CLOCK_PHASE) {
+        data_mode = SPI_MODE2;
+      } else if (CLOCK_POLARITY && CLOCK_PHASE) {
+        data_mode = SPI_MODE3;
+      }
       SPISettings settings(DATA_RATE, BIT_ORDER, data_mode);
       this->hw_spi_->beginTransaction(settings);
     } else {
+#endif  // USE_SPI_ARDUINO_BACKEND
       this->clk_->digital_write(CLOCK_POLARITY);
-      this->wait_cycle_ = uint32_t(F_CPU) / DATA_RATE / 2ULL;
+      uint32_t cpu_freq_hz = arch_get_cpu_freq_hz();
+      this->wait_cycle_ = uint32_t(cpu_freq_hz) / DATA_RATE / 2ULL;
+#ifdef USE_SPI_ARDUINO_BACKEND
     }
+#endif  // USE_SPI_ARDUINO_BACKEND
 
     if (cs != nullptr) {
       this->active_cs_ = cs;
@@ -195,10 +227,6 @@ class SPIComponent : public Component {
  protected:
   inline void cycle_clock_(bool value);
 
-  static void debug_enable(uint8_t pin);
-  static void debug_tx(uint8_t value);
-  static void debug_rx(uint8_t value);
-
   template<SPIBitOrder BIT_ORDER, SPIClockPolarity CLOCK_POLARITY, SPIClockPhase CLOCK_PHASE, bool READ, bool WRITE>
   uint8_t transfer_(uint8_t data);
 
@@ -206,7 +234,9 @@ class SPIComponent : public Component {
   GPIOPin *miso_{nullptr};
   GPIOPin *mosi_{nullptr};
   GPIOPin *active_cs_{nullptr};
+#ifdef USE_SPI_ARDUINO_BACKEND
   SPIClass *hw_spi_{nullptr};
+#endif  // USE_SPI_ARDUINO_BACKEND
   uint32_t wait_cycle_;
 };
 
@@ -246,7 +276,7 @@ class SPIDevice {
     return this->parent_->template write_byte<BIT_ORDER, CLOCK_POLARITY, CLOCK_PHASE>(data);
   }
 
-  void write_byte16(uint8_t data) {
+  void write_byte16(uint16_t data) {
     return this->parent_->template write_byte16<BIT_ORDER, CLOCK_POLARITY, CLOCK_PHASE>(data);
   }
 
